@@ -2,7 +2,7 @@
  *
  * $Id: groupdef.cpp,v 1.29 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -34,6 +34,7 @@
 #include "docparser.h"
 #include "searchindex.h"
 #include "dot.h"
+#include "vhdldocgen.h"
 
 GroupDef::GroupDef(const char *df,int dl,const char *na,const char *t,
                    const char *refFileName) : Definition(df,dl,na)
@@ -45,7 +46,6 @@ GroupDef::GroupDef(const char *df,int dl,const char *na,const char *t,
   pageDict = new PageSDict(17);
   exampleDict = new PageSDict(17);
   dirList = new DirList;
-  allMemberList = new MemberList;
   allMemberNameInfoSDict = new MemberNameInfoSDict(17);
   if (refFileName)
   {
@@ -59,6 +59,9 @@ GroupDef::GroupDef(const char *df,int dl,const char *na,const char *t,
   memberGroupSDict = new MemberGroupSDict;
   memberGroupSDict->setAutoDelete(TRUE);
 
+  allMemberList = new MemberList(MemberList::allMembersList);
+#if 0
+
   decDefineMembers.setInGroup(TRUE);
   decProtoMembers.setInGroup(TRUE);
   decTypedefMembers.setInGroup(TRUE);
@@ -71,10 +74,13 @@ GroupDef::GroupDef(const char *df,int dl,const char *na,const char *t,
   docProtoMembers.setInGroup(TRUE);
   docTypedefMembers.setInGroup(TRUE);
   docEnumMembers.setInGroup(TRUE);
+  docEnumValMembers.setInGroup(TRUE);
   docFuncMembers.setInGroup(TRUE);
   docVarMembers.setInGroup(TRUE);
+#endif
 
   visited = 0;
+  groupScope = 0;
 }
 
 GroupDef::~GroupDef()
@@ -126,40 +132,66 @@ void GroupDef::findSectionsInDocumentation()
   {
     mg->findSectionsInDocumentation();
   }
+
+  QListIterator<MemberList> mli(m_memberLists);
+  MemberList *ml;
+  for (mli.toFirst();(ml=mli.current());++mli)
+  {
+    if (ml->listType()&MemberList::declarationLists)
+    {
+      ml->findSectionsInDocumentation();
+    }
+  }
+#if 0
   decDefineMembers.findSectionsInDocumentation();
   decProtoMembers.findSectionsInDocumentation();
   decTypedefMembers.findSectionsInDocumentation();
   decEnumMembers.findSectionsInDocumentation();
   decFuncMembers.findSectionsInDocumentation();
   decVarMembers.findSectionsInDocumentation();
+#endif
 }
 
 void GroupDef::addFile(const FileDef *def)
 {
+  if (def->isHidden()) return;
   if (Config_getBool("SORT_BRIEF_DOCS"))
     fileList->inSort(def);
   else
     fileList->append(def);
 }
 
-void GroupDef::addClass(const ClassDef *cd)
+bool GroupDef::addClass(const ClassDef *cd)
 {
-  if (Config_getBool("SORT_BRIEF_DOCS"))
-    classSDict->inSort(cd->name(),cd);
-  else
-    classSDict->append(cd->name(),cd);
+  if (cd->isHidden()) return FALSE;
+  if (classSDict->find(cd->name())==0)
+  {
+    if (Config_getBool("SORT_BRIEF_DOCS"))
+      classSDict->inSort(cd->name(),cd);
+    else
+      classSDict->append(cd->name(),cd);
+    return TRUE;
+  }
+  return FALSE;
 }
 
-void GroupDef::addNamespace(const NamespaceDef *def)
+bool GroupDef::addNamespace(const NamespaceDef *def)
 {
-  if (Config_getBool("SORT_BRIEF_DOCS"))
-    namespaceSDict->inSort(def->name(),def);  
-  else
-    namespaceSDict->append(def->name(),def);
+  if (def->isHidden()) return FALSE;
+  if (namespaceSDict->find(def->name())==0)
+  {
+    if (Config_getBool("SORT_BRIEF_DOCS"))
+      namespaceSDict->inSort(def->name(),def);  
+    else
+      namespaceSDict->append(def->name(),def);
+    return TRUE;
+  }
+  return FALSE;
 }
 
 void GroupDef::addDir(const DirDef *def)
 {
+  if (def->isHidden()) return;
   if (Config_getBool("SORT_BRIEF_DOCS"))
     dirList->inSort(def);  
   else
@@ -168,6 +200,7 @@ void GroupDef::addDir(const DirDef *def)
 
 void GroupDef::addPage(PageDef *def)
 {
+  if (def->isHidden()) return;
   //printf("Making page %s part of a group\n",def->name.data());
   pageDict->append(def->name(),def);
   def->makePartOfGroup(this);
@@ -175,19 +208,31 @@ void GroupDef::addPage(PageDef *def)
 
 void GroupDef::addExample(const PageDef *def)
 {
+  if (def->isHidden()) return;
   exampleDict->append(def->name(),def);
 }
 
 
 void GroupDef::addMembersToMemberGroup()
 {
-  ::addMembersToMemberGroup(&decDefineMembers,memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decProtoMembers,memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decTypedefMembers,memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decEnumMembers,memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decEnumValMembers,memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decFuncMembers,memberGroupSDict,this);
-  ::addMembersToMemberGroup(&decVarMembers,memberGroupSDict,this);
+  QListIterator<MemberList> mli(m_memberLists);
+  MemberList *ml;
+  for (mli.toFirst();(ml=mli.current());++mli)
+  {
+    if (ml->listType()&MemberList::declarationLists)
+    {
+      ::addMembersToMemberGroup(ml,&memberGroupSDict,this);
+    }
+  }
+#if 0
+  ::addMembersToMemberGroup(&decDefineMembers,&memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decProtoMembers,&memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decTypedefMembers,&memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decEnumMembers,&memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decEnumValMembers,&memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decFuncMembers,&memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decVarMembers,&memberGroupSDict,this);
+#endif
 
   //printf("GroupDef::addMembersToMemberGroup() memberGroupList=%d\n",memberGroupList->count());
   MemberGroupSDict::Iterator mgli(*memberGroupSDict);
@@ -201,6 +246,7 @@ void GroupDef::addMembersToMemberGroup()
 
 bool GroupDef::insertMember(MemberDef *md,bool docOnly)
 {
+  if (md->isHidden()) return FALSE;
   //printf("GroupDef(%s)::insertMember(%s)\n", title.data(), md->name().data());
   MemberNameInfo *mni=0;
   if ((mni=(*allMemberNameInfoSDict)[md->name()]))
@@ -216,9 +262,15 @@ bool GroupDef::insertMember(MemberDef *md,bool docOnly)
           // both inside a file => definition and declaration do not have to be in the same file
            (srcMd->getOuterScope()->definitionType()==Definition::TypeFile &&
             md->getOuterScope()->definitionType()==Definition::TypeFile); 
+
+      LockingPtr<ArgumentList> srcMdAl = srcMd->argumentList();
+      LockingPtr<ArgumentList> mdAl    = md->argumentList();
       
       if (srcMd->isFunction() && md->isFunction() && 
-          matchArguments(srcMd->argumentList(),md->argumentList()) &&
+          matchArguments2(srcMd->getOuterScope(),srcMd->getFileDef(),srcMdAl.pointer(),
+                          md->getOuterScope(),md->getFileDef(),mdAl.pointer(),
+                          TRUE
+                         ) &&
           sameScope
          )
       {
@@ -248,87 +300,110 @@ bool GroupDef::insertMember(MemberDef *md,bool docOnly)
     case MemberDef::Variable:     
       if (!docOnly)
       {
-        if (Config_getBool("SORT_BRIEF_DOCS"))
-          decVarMembers.inSort(md);
-        else
-          decVarMembers.append(md);
+        addMemberToList(MemberList::decVarMembers,md);
       }
-      if (Config_getBool("SORT_MEMBER_DOCS"))
-        docVarMembers.inSort(md); 
-      else
-        docVarMembers.append(md);
+      addMemberToList(MemberList::docVarMembers,md);
       break;
     case MemberDef::Function: 
       if (!docOnly)
       {
-        if (Config_getBool("SORT_BRIEF_DOCS"))
-          decFuncMembers.inSort(md);
-        else
-          decFuncMembers.append(md);
+        addMemberToList(MemberList::decFuncMembers,md);
       }
-      if (Config_getBool("SORT_MEMBER_DOCS"))    
-        docFuncMembers.inSort(md); 
-      else
-        docFuncMembers.append(md);
+      addMemberToList(MemberList::docFuncMembers,md);
       break;
     case MemberDef::Typedef:      
       if (!docOnly)
       {
-        if (Config_getBool("SORT_BRIEF_DOCS"))
-          decTypedefMembers.inSort(md);
-        else
-          decTypedefMembers.append(md);
+        addMemberToList(MemberList::decTypedefMembers,md);
       }
-      if (Config_getBool("SORT_MEMBER_DOCS"))
-        docTypedefMembers.inSort(md); 
-      else
-        docTypedefMembers.append(md);
+      addMemberToList(MemberList::docTypedefMembers,md);
       break;
     case MemberDef::Enumeration:  
       if (!docOnly)
       {
-        if (Config_getBool("SORT_BRIEF_DOCS"))
-          decEnumMembers.inSort(md);
-        else
-          decEnumMembers.append(md);
+        addMemberToList(MemberList::decEnumMembers,md);
       }
-      if (Config_getBool("SORT_MEMBER_DOCS"))
-        docEnumMembers.inSort(md); 
-      else
-        docEnumMembers.append(md);
+      addMemberToList(MemberList::docEnumMembers,md);
       break;
     case MemberDef::EnumValue:    
+      if (!docOnly)
+      {
+        addMemberToList(MemberList::decEnumValMembers,md);
+      }
+      addMemberToList(MemberList::docEnumValMembers,md);
       break;
     case MemberDef::Prototype:    
       if (!docOnly)
       {
-        if (Config_getBool("SORT_BRIEF_DOCS"))
-          decProtoMembers.inSort(md);
-        else
-          decProtoMembers.append(md);
+        addMemberToList(MemberList::decProtoMembers,md);
       }
-      if (Config_getBool("SORT_MEMBER_DOCS"))
-        docProtoMembers.inSort(md); 
-      else
-        docProtoMembers.append(md);
+      addMemberToList(MemberList::docProtoMembers,md);
       break;
     case MemberDef::Define:       
       if (!docOnly)
       {
-        if (Config_getBool("SORT_BRIEF_DOCS"))
-          decDefineMembers.inSort(md);
-        else
-          decDefineMembers.append(md);
+        addMemberToList(MemberList::decDefineMembers,md);
+      }
+      addMemberToList(MemberList::docDefineMembers,md);
+      break;
+    case MemberDef::Signal:       
+      if (!docOnly)
+      {
+        addMemberToList(MemberList::decSignalMembers,md);
+      }
+      addMemberToList(MemberList::docSignalMembers,md);
+      break;
+    case MemberDef::Slot:       
+      if (md->protection()==Public)
+      {
+        if (!docOnly)
+        {
+          addMemberToList(MemberList::decPubSlotMembers,md);
+        }
+        addMemberToList(MemberList::docPubSlotMembers,md);
+      }
+      else if (md->protection()==Protected)
+      {
+        if (!docOnly)
+        {
+          addMemberToList(MemberList::decProSlotMembers,md);
+        }
+        addMemberToList(MemberList::docProSlotMembers,md);
       }
-      if (Config_getBool("SORT_MEMBER_DOCS"))
-        docDefineMembers.inSort(md); 
       else
-        docDefineMembers.append(md);
+      {
+        if (!docOnly)
+        {
+          addMemberToList(MemberList::decPriSlotMembers,md);
+        }
+        addMemberToList(MemberList::docPriSlotMembers,md);
+      }
+      break;
+    case MemberDef::Event:       
+      if (!docOnly)
+      {
+        addMemberToList(MemberList::decEventMembers,md);
+      }
+      addMemberToList(MemberList::docEventMembers,md);
+      break;
+    case MemberDef::Property:       
+      if (!docOnly)
+      {
+        addMemberToList(MemberList::decPropMembers,md);
+      }
+      addMemberToList(MemberList::docPropMembers,md);
+      break;
+    case MemberDef::Friend:       
+      if (!docOnly)
+      {
+        addMemberToList(MemberList::decFriendMembers,md);
+      }
+      addMemberToList(MemberList::docFriendMembers,md);
       break;
     default:
       err("GroupDef::insertMembers(): "
-           "member `%s' with class scope `%s' inserted in group scope `%s'!\n",
-           md->name().data(),
+           "member `%s' (typeid=%d) with scope `%s' inserted in group scope `%s'!\n",
+           md->name().data(),md->memberType(),
            md->getClassDef() ? md->getClassDef()->name().data() : "",
            name().data());
   }
@@ -357,34 +432,69 @@ void GroupDef::removeMember(MemberDef *md)
       delete mni;
     }
 
-    allMemberList->remove(md); 
+    removeMemberFromList(MemberList::allMembersList,md);
     switch(md->memberType())
     {
       case MemberDef::Variable:
-	decVarMembers.remove(md);
-        docVarMembers.remove(md);
+	removeMemberFromList(MemberList::decVarMembers,md);
+        removeMemberFromList(MemberList::docVarMembers,md);
         break;
       case MemberDef::Function: 
-        decFuncMembers.remove(md);
-        docFuncMembers.remove(md);
+        removeMemberFromList(MemberList::decFuncMembers,md);
+        removeMemberFromList(MemberList::docFuncMembers,md);
         break;
       case MemberDef::Typedef:      
-        decTypedefMembers.remove(md);
-        docTypedefMembers.remove(md);
+        removeMemberFromList(MemberList::decTypedefMembers,md);
+        removeMemberFromList(MemberList::docTypedefMembers,md);
         break;
       case MemberDef::Enumeration:  
-        decEnumMembers.remove(md);
-        docEnumMembers.remove(md);
+        removeMemberFromList(MemberList::decEnumMembers,md);
+        removeMemberFromList(MemberList::docEnumMembers,md);
         break;
       case MemberDef::EnumValue:    
+        removeMemberFromList(MemberList::decEnumValMembers,md);
+        removeMemberFromList(MemberList::docEnumValMembers,md);
         break;
       case MemberDef::Prototype:    
-        decProtoMembers.remove(md);
-        docProtoMembers.remove(md);
+        removeMemberFromList(MemberList::decProtoMembers,md);
+        removeMemberFromList(MemberList::docProtoMembers,md);
         break;
       case MemberDef::Define:       
-        decDefineMembers.remove(md);
-        docDefineMembers.remove(md);
+        removeMemberFromList(MemberList::decDefineMembers,md);
+        removeMemberFromList(MemberList::docDefineMembers,md);
+        break;
+      case MemberDef::Signal:       
+        removeMemberFromList(MemberList::decSignalMembers,md);
+        removeMemberFromList(MemberList::docSignalMembers,md);
+        break;
+      case MemberDef::Slot:       
+        if (md->protection()==Public)
+        {
+          removeMemberFromList(MemberList::decPubSlotMembers,md);
+          removeMemberFromList(MemberList::docPubSlotMembers,md);
+        }
+        else if (md->protection()==Protected)
+        {
+          removeMemberFromList(MemberList::decProSlotMembers,md);
+          removeMemberFromList(MemberList::docProSlotMembers,md);
+        }
+        else
+        {
+          removeMemberFromList(MemberList::decPriSlotMembers,md);
+          removeMemberFromList(MemberList::docPriSlotMembers,md);
+        }
+        break;
+      case MemberDef::Event:       
+        removeMemberFromList(MemberList::decEventMembers,md);
+        removeMemberFromList(MemberList::docEventMembers,md);
+        break;
+      case MemberDef::Property:       
+        removeMemberFromList(MemberList::decPropMembers,md);
+        removeMemberFromList(MemberList::docPropMembers,md);
+        break;
+      case MemberDef::Friend:       
+        removeMemberFromList(MemberList::decFriendMembers,md);
+        removeMemberFromList(MemberList::docFriendMembers,md);
         break;
       default:
         err("GroupDef::removeMember(): unexpected member remove in file!\n");
@@ -394,7 +504,7 @@ void GroupDef::removeMember(MemberDef *md)
 
 bool GroupDef::containsGroup(const GroupDef *def)
 {
-    return groupList->find(def) >= 0;
+    return this==def || groupList->find(def) >= 0;
 }
 
 void GroupDef::addGroup(const GroupDef *def)
@@ -408,7 +518,8 @@ void GroupDef::addGroup(const GroupDef *def)
 
 bool GroupDef::isASubGroup() const
 {
-  return m_partOfGroups && m_partOfGroups->count()!=0;
+  LockingPtr<GroupList> groups = partOfGroups();
+  return groups!=0 && groups->count()!=0;
 }
 
 int GroupDef::countMembers() const
@@ -465,6 +576,8 @@ void GroupDef::writeDetailedDocumentation(OutputList &ol)
 
 void GroupDef::writeDocumentation(OutputList &ol)
 {
+  bool fortranOpt = Config_getBool("OPTIMIZE_FOR_FORTRAN");  
+  bool vhdlOpt    = Config_getBool("OPTIMIZE_OUTPUT_VHDL");  
   ol.pushGeneratorState();
   startFile(ol,getOutputFileBase(),name(),title);
   startTitle(ol,getOutputFileBase());
@@ -645,13 +758,23 @@ void GroupDef::writeDocumentation(OutputList &ol)
       mg->writeDeclarations(ol,0,0,0,this);
     }
 
-    //allMemberList->writeDeclarations(ol,0,0,0,this,0,0); 
-    decDefineMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trDefines(),0);
-    decProtoMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trFuncProtos(),0);
-    decTypedefMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trTypedefs(),0);
-    decEnumMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trEnumerations(),0);
-    decFuncMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trFunctions(),0);
-    decVarMembers.writeDeclarations(ol,0,0,0,this,theTranslator->trVariables(),0);
+    writeMemberDeclarations(ol,MemberList::decDefineMembers,theTranslator->trDefines());
+    writeMemberDeclarations(ol,MemberList::decProtoMembers,theTranslator->trFuncProtos());
+    writeMemberDeclarations(ol,MemberList::decTypedefMembers,theTranslator->trTypedefs());
+    writeMemberDeclarations(ol,MemberList::decEnumMembers,theTranslator->trEnumerations());
+    writeMemberDeclarations(ol,MemberList::decEnumValMembers,theTranslator->trEnumerationValues());
+    writeMemberDeclarations(ol,MemberList::decFuncMembers,
+                 fortranOpt ? theTranslator->trSubprograms()  : 
+                 vhdlOpt    ? VhdlDocGen::trFunctionAndProc() :
+                              theTranslator->trFunctions());
+    writeMemberDeclarations(ol,MemberList::decVarMembers,theTranslator->trVariables());
+    writeMemberDeclarations(ol,MemberList::decSignalMembers,theTranslator->trSignals());
+    writeMemberDeclarations(ol,MemberList::decPubSlotMembers,theTranslator->trPublicSlots());
+    writeMemberDeclarations(ol,MemberList::decProSlotMembers,theTranslator->trProtectedSlots());
+    writeMemberDeclarations(ol,MemberList::decPriSlotMembers,theTranslator->trPrivateSlots());
+    writeMemberDeclarations(ol,MemberList::decEventMembers,theTranslator->trEvents());
+    writeMemberDeclarations(ol,MemberList::decPropMembers,theTranslator->trProperties());
+    writeMemberDeclarations(ol,MemberList::decFriendMembers,theTranslator->trFriends());
   }
   ol.endMemberSections();
 
@@ -687,23 +810,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
     }
   }
 
-  docDefineMembers.writeDocumentation(ol,name(),this,
-                             theTranslator->trDefineDocumentation());
-  
-  docProtoMembers.writeDocumentation(ol,name(),this,
-                             theTranslator->trFunctionPrototypeDocumentation());
-
-  docTypedefMembers.writeDocumentation(ol,name(),this,
-                             theTranslator->trTypedefDocumentation());
-  
-  docEnumMembers.writeDocumentation(ol,name(),this,
-                             theTranslator->trEnumerationTypeDocumentation());
-
-  docFuncMembers.writeDocumentation(ol,name(),this,
-                             theTranslator->trFunctionDocumentation());
-  
-  docVarMembers.writeDocumentation(ol,name(),this,
-                             theTranslator->trVariableDocumentation());
+  writeMemberDocumentation(ol);
 
   if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
   {
@@ -713,8 +820,111 @@ void GroupDef::writeDocumentation(OutputList &ol)
 
   endFile(ol); 
   ol.popGeneratorState();
+
+  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
+  {
+    allMemberList->sort();
+    writeMemberPages(ol);
+  }
+
 }
 
+void GroupDef::writeMemberDocumentation(OutputList &ol)
+{
+  bool  fortranOpt = Config_getBool("OPTIMIZE_FOR_FORTRAN");
+  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
+  {
+    ol.disable(OutputGenerator::Html);
+  }
+  
+  writeMemberDocumentation(ol,MemberList::docDefineMembers,theTranslator->trDefineDocumentation());
+  writeMemberDocumentation(ol,MemberList::docProtoMembers,theTranslator->trFunctionPrototypeDocumentation());
+  writeMemberDocumentation(ol,MemberList::docTypedefMembers,theTranslator->trTypedefDocumentation());
+  writeMemberDocumentation(ol,MemberList::docEnumMembers,theTranslator->trEnumerationTypeDocumentation());
+  writeMemberDocumentation(ol,MemberList::docEnumValMembers,theTranslator->trEnumerationValueDocumentation());
+  writeMemberDocumentation(ol,MemberList::docFuncMembers,fortranOpt?theTranslator->trSubprogramDocumentation():theTranslator->trFunctionDocumentation());
+  writeMemberDocumentation(ol,MemberList::docVarMembers,theTranslator->trVariableDocumentation());
+  writeMemberDocumentation(ol,MemberList::docSignalMembers,theTranslator->trSignals());   // todo: add trSignalDocumentation()
+  writeMemberDocumentation(ol,MemberList::docPubSlotMembers,theTranslator->trPublicSlots()); // todo: add trSlotDocumentation()
+  writeMemberDocumentation(ol,MemberList::docProSlotMembers,theTranslator->trProtectedSlots()); // todo: add trSlotDocumentation()
+  writeMemberDocumentation(ol,MemberList::docPriSlotMembers,theTranslator->trPrivateSlots()); // todo: add trSlotDocumentation()
+  writeMemberDocumentation(ol,MemberList::docEventMembers,theTranslator->trEvents());     // todo: add trEventDocumentation()
+  writeMemberDocumentation(ol,MemberList::docPropMembers,theTranslator->trProperties());  // todo: add trPropertyDocumentation()
+  writeMemberDocumentation(ol,MemberList::docFriendMembers,theTranslator->trFriends());   // todo: add trFriendDocumentation()
+
+  if (Config_getBool("SEPARATE_MEMBER_PAGES"))
+  {
+    ol.enable(OutputGenerator::Html);
+  }
+}
+
+void GroupDef::writeMemberPages(OutputList &ol)
+{
+  ol.pushGeneratorState();
+  ol.disableAllBut(OutputGenerator::Html);
+  
+  QListIterator<MemberList> mli(m_memberLists);
+  MemberList *ml;
+  for (mli.toFirst();(ml=mli.current());++mli)
+  {
+    if (ml->listType()&MemberList::documentationLists)
+    {
+       ml->writeDocumentationPage(ol,name(),this);
+    }
+  }
+#if 0
+  docDefineMembers.writeDocumentationPage(ol,name(),this);
+  docProtoMembers.writeDocumentationPage(ol,name(),this);
+  docTypedefMembers.writeDocumentationPage(ol,name(),this);
+  docEnumMembers.writeDocumentationPage(ol,name(),this);
+  docFuncMembers.writeDocumentationPage(ol,name(),this);
+  docVarMembers.writeDocumentationPage(ol,name(),this);
+#endif
+
+  ol.popGeneratorState();
+}
+
+void GroupDef::writeQuickMemberLinks(OutputList &ol,MemberDef *currentMd) const
+{
+  static bool createSubDirs=Config_getBool("CREATE_SUBDIRS");
+
+  ol.writeString("      <div class=\"navtab\">\n");
+  ol.writeString("        <table>\n");
+
+  MemberListIterator mli(*allMemberList);
+  MemberDef *md;
+  for (mli.toFirst();(md=mli.current());++mli)
+  {
+    if (md->getGroupDef()==this && md->isLinkable())
+    {
+      ol.writeString("          <tr><td class=\"navtab\">");
+      if (md->isLinkableInProject())
+      {
+        if (md==currentMd) // selected item => highlight
+        {
+          ol.writeString("<a class=\"qindexHL\" ");
+        }
+        else
+        {
+          ol.writeString("<a class=\"qindex\" ");
+        }
+        ol.writeString("href=\"");
+        if (createSubDirs) ol.writeString("../../");
+        ol.writeString(md->getOutputFileBase()+Doxygen::htmlFileExtension+"#"+md->anchor());
+        ol.writeString("\">");
+        ol.writeString(md->localName());
+        ol.writeString("</a>");
+      }
+      ol.writeString("</td></tr>\n");
+    }
+  }
+
+  ol.writeString("        </table>\n");
+  ol.writeString("      </div>\n");
+}
+
+
+
 //---- helper functions ------------------------------------------------------
 
 void addClassToGroups(Entry *root,ClassDef *cd)
@@ -724,10 +934,9 @@ void addClassToGroups(Entry *root,ClassDef *cd)
   for (;(g=gli.current());++gli)
   {
     GroupDef *gd=0;
-    if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict[g->groupname]))
+    if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g->groupname)))
     {
-      gd->addClass(cd);
-      cd->makePartOfGroup(gd);
+      if (gd->addClass(cd)) cd->makePartOfGroup(gd);
       //printf("Compound %s: in group %s\n",cd->name().data(),s->data());
     }
   }
@@ -742,10 +951,9 @@ void addNamespaceToGroups(Entry *root,NamespaceDef *nd)
   {
     GroupDef *gd=0;
     //printf("group `%s'\n",s->data());
-    if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict[g->groupname]))
+    if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g->groupname)))
     {
-      gd->addNamespace(nd);
-      nd->makePartOfGroup(gd);
+      if (gd->addNamespace(nd)) nd->makePartOfGroup(gd);
       //printf("Namespace %s: in group %s\n",nd->name().data(),s->data());
     }
   }
@@ -760,7 +968,7 @@ void addDirToGroups(Entry *root,DirDef *dd)
   {
     GroupDef *gd=0;
     //printf("group `%s'\n",g->groupname.data());
-    if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict[g->groupname]))
+    if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g->groupname)))
     {
       gd->addDir(dd);
       dd->makePartOfGroup(gd);
@@ -771,17 +979,24 @@ void addDirToGroups(Entry *root,DirDef *dd)
 
 void addGroupToGroups(Entry *root,GroupDef *subGroup)
 {
+  //printf("addGroupToGroups for %s groups=%d\n",root->name.data(),
+  //    root->groups?root->groups->count():-1);
   QListIterator<Grouping> gli(*root->groups);
   Grouping *g;
   for (;(g=gli.current());++gli)
   {
     GroupDef *gd=0;
-    if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict[g->groupname]) &&
+    if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g->groupname)) &&
 	!gd->containsGroup(subGroup) )
     {
       gd->addGroup(subGroup);
       subGroup->makePartOfGroup(gd);
     }
+    else if (gd==subGroup)
+    {
+      warn(root->fileName,root->startLine,"Trying to add group %s to itself!",
+          gd->name().data());
+    }
   }
 }
 
@@ -800,10 +1015,10 @@ void addMemberToGroups(Entry *root,MemberDef *md)
   {
     GroupDef *gd=0;
     if (!g->groupname.isEmpty() &&
-        (gd=Doxygen::groupSDict[g->groupname]) &&
+        (gd=Doxygen::groupSDict->find(g->groupname)) &&
 	g->pri >= pri)
     {
-      if (fgd && g->pri==pri) 
+      if (fgd && gd!=fgd && g->pri==pri) 
       {
          warn(root->fileName.data(), root->startLine,
            "Warning: Member %s found in multiple %s groups! "
@@ -894,7 +1109,7 @@ void addExampleToGroups(Entry *root,PageDef *eg)
   for (;(g=gli.current());++gli)
   {
     GroupDef *gd=0;
-    if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict[g->groupname]))
+    if (!g->groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g->groupname)))
     {
       gd->addExample(eg);
       eg->makePartOfGroup(gd);
@@ -917,22 +1132,100 @@ QCString GroupDef::getOutputFileBase() const
 
 void GroupDef::addListReferences()
 {
-  addRefItem(xrefListItems(),
+  {
+    LockingPtr< QList<ListItemInfo> > xrefItems = xrefListItems();
+    addRefItem(xrefItems.pointer(),
              theTranslator->trGroup(TRUE,TRUE),
              getOutputFileBase(),name()
             );
+  }
   MemberGroupSDict::Iterator mgli(*memberGroupSDict);
   MemberGroup *mg;
   for (;(mg=mgli.current());++mgli)
   {
     mg->addListReferences(this);
   }
+  QListIterator<MemberList> mli(m_memberLists);
+  MemberList *ml;
+  for (mli.toFirst();(ml=mli.current());++mli)
+  {
+    if (ml->listType()&MemberList::documentationLists)
+    {
+      ml->addListReferences(this);
+    }
+  }
+#if 0
   docDefineMembers.addListReferences(this);
   docProtoMembers.addListReferences(this);
   docTypedefMembers.addListReferences(this);
   docEnumMembers.addListReferences(this);
   docFuncMembers.addListReferences(this);
   docVarMembers.addListReferences(this);
+#endif
 }
 
+MemberList *GroupDef::createMemberList(MemberList::ListType lt)
+{
+  m_memberLists.setAutoDelete(TRUE);
+  QListIterator<MemberList> mli(m_memberLists);
+  MemberList *ml;
+  for (mli.toFirst();(ml=mli.current());++mli)
+  {
+    if (ml->listType()==lt)
+    {
+      return ml;
+    }
+  }
+  // not found, create a new member list
+  ml = new MemberList(lt);
+  m_memberLists.append(ml);
+  ml->setInGroup(TRUE);
+  return ml;
+}
+
+void GroupDef::addMemberToList(MemberList::ListType lt,MemberDef *md)
+{
+  static bool sortBriefDocs = Config_getBool("SORT_BRIEF_DOCS");
+  static bool sortMemberDocs = Config_getBool("SORT_MEMBER_DOCS");
+  MemberList *ml = createMemberList(lt);
+  if (((ml->listType()&MemberList::declarationLists) && sortBriefDocs) ||
+      ((ml->listType()&MemberList::documentationLists) && sortMemberDocs)
+     )
+    ml->inSort(md);
+  else
+    ml->append(md);
+}
+
+MemberList *GroupDef::getMemberList(MemberList::ListType lt) const
+{
+  GroupDef *that = (GroupDef*)this;
+  MemberList *ml = that->m_memberLists.first();
+  while (ml)
+  {
+    if (ml->listType()==lt)
+    {
+      return ml;
+    }
+    ml = that->m_memberLists.next();
+  }
+  return 0;
+}
+
+void GroupDef::writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title)
+{
+  MemberList * ml = getMemberList(lt);
+  if (ml) ml->writeDeclarations(ol,0,0,0,this,title,0);
+}
+
+void GroupDef::writeMemberDocumentation(OutputList &ol,MemberList::ListType lt,const QCString &title)
+{
+  MemberList * ml = getMemberList(lt);
+  if (ml) ml->writeDocumentation(ol,name(),this,title);
+}
+
+void GroupDef::removeMemberFromList(MemberList::ListType lt,MemberDef *md)
+{
+    MemberList *ml = getMemberList(lt);
+    if (ml) ml->remove(md); 
+}
 
