@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,10 +21,11 @@
 #include "language.h"
 #include "doxygen.h"
 #include "outputgen.h"
-#include "code.h"
 #include "dot.h"
 #include "util.h"
 #include "message.h"
+#include "parserintf.h"
+#include "msc.h"
 
 static QString escapeLabelName(const char *s)
 {
@@ -79,10 +80,11 @@ QString LatexDocVisitor::escapeMakeIndexChars(const char *s)
 }
 
 
-LatexDocVisitor::LatexDocVisitor(QTextStream &t,BaseCodeDocInterface &ci,
-                                 bool insideTabbing) 
+LatexDocVisitor::LatexDocVisitor(QTextStream &t,CodeOutputInterface &ci,
+                                 const char *langExt,bool insideTabbing) 
   : DocVisitor(DocVisitor_Latex), m_t(t), m_ci(ci), m_insidePre(FALSE), 
-    m_insideItem(FALSE), m_hide(FALSE), m_insideTabbing(insideTabbing) 
+    m_insideItem(FALSE), m_hide(FALSE), m_insideTabbing(insideTabbing),
+    m_langExt(langExt)
 {
 }
 
@@ -124,8 +126,9 @@ void LatexDocVisitor::visit(DocSymbol *s)
   {
     case DocSymbol::BSlash:  m_t << "$\\backslash$"; break;
     case DocSymbol::At:      m_t << "@"; break;
-    case DocSymbol::Less:    m_t << "$<$"; break;
-    case DocSymbol::Greater: m_t << "$>$"; break;
+    case DocSymbol::Less:    if (m_insidePre) m_t << "<"; else m_t << "$<$"; 
+                             break;
+    case DocSymbol::Greater: if (m_insidePre) m_t << ">"; else m_t << "$>$"; break;
     case DocSymbol::Amp:     m_t << "\\&"; break;
     case DocSymbol::Dollar:  m_t << "\\$"; break;
     case DocSymbol::Hash:    m_t << "\\#"; break;
@@ -134,7 +137,13 @@ void LatexDocVisitor::visit(DocSymbol *s)
     case DocSymbol::Tm:      m_t << "\\texttrademark"; break;
     case DocSymbol::Reg:     m_t << "\\textregistered"; break;
     case DocSymbol::Apos:    m_t << "'"; break;
-    case DocSymbol::Quot:    m_t << "''"; break;
+    case DocSymbol::Quot:    m_t << "\""; break;
+    case DocSymbol::Lsquo:   m_t << "`"; break;
+    case DocSymbol::Rsquo:   m_t << "'"; break;
+    case DocSymbol::Ldquo:   m_t << "``"; break;
+    case DocSymbol::Rdquo:   m_t << "''"; break;
+    case DocSymbol::Ndash:   m_t << "--"; break;
+    case DocSymbol::Mdash:   m_t << "---"; break;
     case DocSymbol::Uml:     
                              if (s->letter()=='i') 
                                m_t << "\\\"{\\i}"; 
@@ -165,10 +174,10 @@ void LatexDocVisitor::visit(DocSymbol *s)
                                m_t << s->letter();
                              break;
     case DocSymbol::Tilde:   m_t << "\\~{"  << s->letter() << "}"; break;
-    case DocSymbol::Szlig:   m_t << "\"s"; break;
+    case DocSymbol::Szlig:   m_t << "{\\ss}"; break;
     case DocSymbol::Cedil:   m_t << "\\c{" << s->letter() << "}"; break;
     case DocSymbol::Ring:    m_t << "\\" << s->letter() << s->letter(); break;
-    case DocSymbol::Nbsp:    m_t << "\\ "; break;
+    case DocSymbol::Nbsp:    m_t << "~"; break;
     default:
                              err("Error: unknown symbol found\n");
   }
@@ -224,7 +233,7 @@ void LatexDocVisitor::visit(DocStyleChange *s)
       if (s->enable()) m_t << "\\begin{center}"; else m_t << "\\end{center} ";
       break;
     case DocStyleChange::Small:
-      if (s->enable()) m_t << "\\footnotesize ";  else m_t << "\\normalsize ";
+      if (s->enable()) m_t << "\n\\footnotesize ";  else m_t << "\n\\normalsize ";
       break;
     case DocStyleChange::Preformatted:
       if (s->enable()) 
@@ -235,7 +244,7 @@ void LatexDocVisitor::visit(DocStyleChange *s)
       else
       {
         m_insidePre=FALSE;
-        m_t << "\\end{alltt}\\normalsize " << endl;
+        m_t << "\\end{alltt}\n\\normalsize " << endl;
       }
       break;
     case DocStyleChange::Div:  /* HTML only */ break;
@@ -249,9 +258,11 @@ void LatexDocVisitor::visit(DocVerbatim *s)
   switch(s->type())
   {
     case DocVerbatim::Code: 
-      m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
-      parseCode(m_ci,s->context(),s->text().latin1(),s->isExample(),s->exampleFile());
-      m_t << "\\end{verbatim}\n\\normalsize" << endl; 
+      m_t << "\n\n\\begin{Code}\\begin{verbatim}"; 
+      Doxygen::parserManager->getParser(m_langExt)
+                            ->parseCode(m_ci,s->context(),s->text().latin1(),
+                                        s->isExample(),s->exampleFile());
+      m_t << "\\end{verbatim}\n\\end{Code}\n" << endl; 
       break;
     case DocVerbatim::Verbatim: 
       m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
@@ -271,9 +282,10 @@ void LatexDocVisitor::visit(DocVerbatim *s)
         static int dotindex = 1;
         QCString fileName(4096);
 
-        fileName.sprintf("%s%d", 
+        fileName.sprintf("%s%d%s", 
             (Config_getString("LATEX_OUTPUT")+"/inline_dotgraph_").data(), 
-            dotindex++
+            dotindex++,
+            ".dot"
            );
         QFile file(fileName);
         if (!file.open(IO_WriteOnly))
@@ -288,7 +300,34 @@ void LatexDocVisitor::visit(DocVerbatim *s)
         endDotFile(FALSE);
         m_t << "\\end{center}\n";
 
-        file.remove();
+        if (Config_getBool("DOT_CLEANUP")) file.remove();
+      }
+      break;
+    case DocVerbatim::Msc: 
+      {
+        static int mscindex = 1;
+        QCString baseName(4096);
+
+        baseName.sprintf("%s%d", 
+            (Config_getString("LATEX_OUTPUT")+"/inline_mscgraph_").data(), 
+            mscindex++
+           );
+        QFile file(baseName+".msc");
+        if (!file.open(IO_WriteOnly))
+        {
+          err("Could not open file %s.msc for writing\n",baseName.data());
+        }
+        QCString text = "msc {";
+        text+=s->text();
+        text+="}";
+        file.writeBlock( text, text.length() );
+        file.close();
+
+        m_t << "\\begin{center}\n";
+        writeMscFile(baseName);
+        m_t << "\\end{center}\n";
+
+        if (Config_getBool("DOT_CLEANUP")) file.remove();
       }
       break;
   }
@@ -312,26 +351,33 @@ void LatexDocVisitor::visit(DocInclude *inc)
   {
     case DocInclude::IncWithLines:
       { 
-         m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
+         m_t << "\n\n\\begin{DocInclude}\\begin{verbatim}"; 
          QFileInfo cfi( inc->file() );
          FileDef fd( cfi.dirPath(), cfi.fileName() );
-         parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile(), &fd);
-         m_t << "\\end{verbatim}\n\\normalsize" << endl; 
+         Doxygen::parserManager->getParser(inc->extension())
+                               ->parseCode(m_ci,inc->context(),
+                                           inc->text().latin1(),
+                                           inc->isExample(),
+                                           inc->exampleFile(), &fd);
+         m_t << "\\end{verbatim}\n\\end{DocInclude}" << endl; 
       }
       break;    
     case DocInclude::Include: 
-      m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
-      parseCode(m_ci,inc->context(),inc->text().latin1(),inc->isExample(),inc->exampleFile());
-      m_t << "\\end{verbatim}\n\\normalsize" << endl; 
+      m_t << "\n\n\\begin{DocInclude}\\begin{verbatim}"; 
+      Doxygen::parserManager->getParser(inc->extension())
+                            ->parseCode(m_ci,inc->context(),
+                                        inc->text().latin1(),inc->isExample(),
+                                        inc->exampleFile());
+      m_t << "\\end{verbatim}\n\\end{DocInclude}" << endl; 
       break;
     case DocInclude::DontInclude: 
       break;
     case DocInclude::HtmlInclude: 
       break;
     case DocInclude::VerbInclude: 
-      m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
+      m_t << "\n\n\\begin{VerbInclude}\\begin{verbatim}"; 
       m_t << inc->text();
-      m_t << "\\end{verbatim}\n\\normalsize" << endl; 
+      m_t << "\\end{verbatim}\n\\end{VerbInclude}" << endl; 
       break;
   }
 }
@@ -342,21 +388,26 @@ void LatexDocVisitor::visit(DocIncOperator *op)
   //    op->type(),op->isFirst(),op->isLast(),op->text().data());
   if (op->isFirst()) 
   {
-    if (!m_hide) m_t << "\n\n\\footnotesize\\begin{verbatim}"; 
+    if (!m_hide) m_t << "\n\n\\begin{DocInclude}\\begin{verbatim}"; 
     pushEnabled();
     m_hide = TRUE;
   }
   if (op->type()!=DocIncOperator::Skip) 
   {
     popEnabled();
-    if (!m_hide) parseCode(m_ci,op->context(),op->text().latin1(),op->isExample(),op->exampleFile());
+    if (!m_hide) 
+    {
+      Doxygen::parserManager->getParser(m_langExt)
+                            ->parseCode(m_ci,op->context(),op->text().latin1(),
+                                        op->isExample(),op->exampleFile());
+    }
     pushEnabled();
     m_hide=TRUE;
   }
   if (op->isLast())  
   {
     popEnabled();
-    if (!m_hide) m_t << "\\end{verbatim}\n\\normalsize" << endl; 
+    if (!m_hide) m_t << "\\end{verbatim}\n\\end{DocInclude}" << endl; 
   }
   else
   {
@@ -537,7 +588,7 @@ void LatexDocVisitor::visitPre(DocSection *s)
     m_t << "\\hypertarget{" << s->file() << "_" << s->anchor() << "}{}";
   }
   m_t << "\\" << getSectionName(s->level()) << "{";
-  filter(s->title());
+  filter(convertCharEntitiesToUTF8(s->title().data()));
   m_t << "}\\label{" << s->file() << "_" << s->anchor() << "}" << endl;
 }
 
@@ -724,12 +775,12 @@ void LatexDocVisitor::visitPre(DocImage *img)
     if (m_hide) return;
     if (img->hasCaption())
     {
-      m_t << "\\begin{figure}[H]" << endl;
+      m_t << "\\begin{Image}" << endl;
       m_t << "\\begin{center}" << endl;
     }
     else
     {
-      m_t << "\\mbox{";
+      m_t << "\\begin{ImageNoCaption}\\mbox{";
     }
     QString gfxName = img->name();
     if (gfxName.right(4)==".eps" || gfxName.right(4)==".pdf")
@@ -767,7 +818,10 @@ void LatexDocVisitor::visitPost(DocImage *img)
     if (img->hasCaption())
     {
       m_t << "\\end{center}" << endl;
-      m_t << "\\end{figure}" << endl;
+      m_t << "\\end{Image}" << endl;
+    }
+    else{
+      m_t << "\\end{ImageNoCaption}" << endl;
     }
   }
   else // other format
@@ -873,6 +927,11 @@ void LatexDocVisitor::visitPre(DocParamSect *s)
       filter(theTranslator->trReturnValues()); break;
     case DocParamSect::Exception: 
       filter(theTranslator->trExceptions()); break;
+    case DocParamSect::TemplateParam: 
+      /* TODO: add this 
+      filter(theTranslator->trTemplateParam()); break;
+      */
+      filter("Template Parameters"); break;
     default:
       ASSERT(0);
   }
@@ -909,14 +968,23 @@ void LatexDocVisitor::visitPre(DocParamList *pl)
     m_t << "} ";
   }
   m_t << "{\\em ";
-  QStrListIterator li(pl->parameters());
-  const char *s;
+  //QStrListIterator li(pl->parameters());
+  //const char *s;
+  QListIterator<DocNode> li(pl->parameters());
+  DocNode *param;
   bool first=TRUE;
-  for (li.toFirst();(s=li.current());++li)
+  for (li.toFirst();(param=li.current());++li)
   {
     if (!first) m_t << ","; else first=FALSE;
     m_insideItem=TRUE;
-    filter(s);
+    if (param->kind()==DocNode::Kind_Word)
+    {
+      visit((DocWord*)param); 
+    }
+    else if (param->kind()==DocNode::Kind_LinkedWord)
+    {
+      visit((DocLinkedWord*)param); 
+    }
     m_insideItem=FALSE;
   }
   m_t << "}]";
@@ -987,17 +1055,31 @@ void LatexDocVisitor::filter(const char *str)
 
 void LatexDocVisitor::startLink(const QString &ref,const QString &file,const QString &anchor)
 {
-  if (ref.isEmpty() && Config_getBool("PDF_HYPERLINKS"))
+  if (ref.isEmpty() && Config_getBool("PDF_HYPERLINKS")) // internal PDF link 
   {
-    m_t << "\\hyperlink{";
-    if (!file.isEmpty()) m_t << stripPath(file);
-    if (!file.isEmpty() && !anchor.isEmpty()) m_t << "_";
-    if (!anchor.isEmpty()) m_t << anchor;
-    m_t << "}{";
-
+    if (ref.isEmpty()) {
+      m_t << "\\hyperlink{";
+      if (!file.isEmpty()) m_t << stripPath(file);
+      if (!file.isEmpty() && !anchor.isEmpty()) m_t << "_";
+      if (!anchor.isEmpty()) m_t << anchor;
+      m_t << "}{";
+    }
+    else
+    {
+      QCString *dest;
+      m_t << "\\href{";
+      if ((dest=Doxygen::tagDestinationDict[ref])) m_t << *dest << "/";
+      if (!file.isEmpty()) m_t << file << Doxygen::htmlFileExtension;
+      if (!anchor.isEmpty()) m_t << "#" << anchor;
+      m_t << "}{";
+    }
   }
-  else
+  else if (ref.isEmpty()) // internal non-PDF link
   {
+    m_t << "\\doxyref{";
+  }
+  else // external link
+  { 
     m_t << "{\\bf ";
   }
 }
@@ -1007,11 +1089,11 @@ void LatexDocVisitor::endLink(const QString &ref,const QString &file,const QStri
   m_t << "}";
   if (ref.isEmpty() && !Config_getBool("PDF_HYPERLINKS"))
   {
-    m_t << "{\\rm ("; 
+    m_t << "{"; 
     filter(theTranslator->trPageAbbreviation());
-    m_t << "\\,\\pageref{" << file;
+    m_t << "}{" << file;
     if (!anchor.isEmpty()) m_t << "_" << anchor;
-    m_t << "})}";
+    m_t << "}";
   }
 }
 
@@ -1053,12 +1135,12 @@ void LatexDocVisitor::startDotFile(const QString &fileName,
   writeDotGraphFromFile(name,outDir,baseName,EPS);
   if (hasCaption)
   {
-    m_t << "\\begin{figure}[H]" << endl;
+    m_t << "\\begin{Image}" << endl;
     m_t << "\\begin{center}" << endl;
   }
   else
   {
-    m_t << "\\mbox{";
+    m_t << "\\begin{ImageNoCaption}\\mbox{";
   }
   m_t << "\\includegraphics";
   if (!width.isEmpty())
@@ -1080,11 +1162,35 @@ void LatexDocVisitor::startDotFile(const QString &fileName,
 void LatexDocVisitor::endDotFile(bool hasCaption)
 {
   if (m_hide) return;
-  m_t << "}" << endl; // end mbox or caption
+  m_t << "}" << endl; // end caption or mbox
   if (hasCaption)
   {
     m_t << "\\end{center}" << endl;
-    m_t << "\\end{figure}" << endl;
+    m_t << "\\end{Image}" << endl;
+  }
+  else
+  {
+    m_t << "\\end{ImageNoCaption}" << endl;
   }
 }
 
+void LatexDocVisitor::writeMscFile(const QString &baseName)
+{
+  QString outDir = Config_getString("LATEX_OUTPUT");
+  writeMscGraphFromFile(baseName,outDir,baseName,MSC_EPS);
+  m_t << "\\begin{ImageNoCaption}\\mbox{";
+  m_t << "\\includegraphics";
+  //if (!width.isEmpty())
+  //{
+  //  m_t << "[width=" << width << "]";
+  //}
+  //else if (!height.isEmpty())
+  //{
+  //  m_t << "[height=" << height << "]";
+  //}
+  m_t << "{" << baseName << "}";
+
+  m_t << "}" << endl; // end mbox
+  m_t << "\\end{ImageNoCaption}" << endl;
+}
+
