@@ -2,7 +2,7 @@
  *
  *
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2008 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby
@@ -29,39 +29,39 @@
 #ifndef TRANSLATOR_JP_H
 #define TRANSLATOR_JP_H
 
-class TranslatorJapanese : public TranslatorAdapter_1_3_9
+class TranslatorJapanese : public TranslatorAdapter_1_5_4
 {
  private:
   /*! The decode() can change euc into sjis */
   inline QCString decode(const QCString & sInput)
   {
-    if (Config_getBool("USE_WINDOWS_ENCODING"))
-    {
-      return JapaneseEucToSjis(sInput);
-    }
-    else
-    {
+    //if (Config_getBool("USE_WINDOWS_ENCODING"))
+    //{
+    //  return JapaneseEucToSjis(sInput);
+    //}
+    //else
+    //{
       return sInput;
-    }
+    //}
   }
   public:
     virtual QCString idLanguage()
     { return "japanese"; }
     virtual QCString latexLanguageSupportCommand()
-      {
-	return "";
-      }
+    {
+      return "";
+    }
     /*! returns the name of the package that is included by LaTeX */
     virtual QCString idLanguageCharset()
     {
-      if (Config_getBool("USE_WINDOWS_ENCODING"))
-      {
-        return "Shift_JIS";
-      }
-      else
-      {
+      //if (Config_getBool("USE_WINDOWS_ENCODING"))
+      //{
+      //  return "Shift_JIS";
+      //}
+      //else
+      //{
         return "euc-jp";
-      }
+      //}
     }
 
     /*! used in the compound documentation before a list of related functions. */
@@ -453,12 +453,6 @@ class TranslatorJapanese : public TranslatorAdapter_1_3_9
     virtual QCString trEnumerationTypeDocumentation()
     { return decode(""); }
 
-    /*! This is used in the documentation of a file/namespace before the list
-     *  of documentation blocks for enumeration values
-     */
-    virtual QCString trEnumerationValueDocumentation()
-    { return decode("󷿤"); }
-
     /*! This is used in the documentation of a file/namespace before the list
      *  of documentation blocks for functions
      */
@@ -661,7 +655,12 @@ class TranslatorJapanese : public TranslatorAdapter_1_3_9
                                    // (order is left to right)
 
         if (i!=numEntries-1)  // not the last entry, so we need a separator
-	  result+=decode("");
+        {
+          if (i<numEntries-2) // not the fore last entry
+            result+=decode(", ");
+          else                // the fore last entry
+            result+=decode(",  ");
+        }
       }
       return result;
     }
@@ -1002,6 +1001,8 @@ class TranslatorJapanese : public TranslatorAdapter_1_3_9
         "class Undocumented { };\n\n"
         "/*! public ǷѾ줿饹 */\n"
         "class PublicBase : public Truncated { };\n\n"
+        "/*! A template class */\n"
+        "template<class T> class Templ { };\n\n"
         "/*! protected ǷѾ줿饹 */\n"
         "class ProtectedBase { };\n\n"
         "/*! private ǷѾ줿饹 */\n"
@@ -1012,7 +1013,8 @@ class TranslatorJapanese : public TranslatorAdapter_1_3_9
         "class Inherited : public PublicBase,\n"
         "                  protected ProtectedBase,\n"
         "                  private PrivateBase,\n"
-        "                  public Undocumented\n"
+        "                  public Undocumented,\n"
+        "                  public Templ<int>\n"
         "{\n"
         "  private:\n"
         "    Used *m_usedClass;\n"
@@ -1562,5 +1564,23 @@ class TranslatorJapanese : public TranslatorAdapter_1_3_9
 		     "ΤѰդƤޤ"
 		     "δؿȤΰ㤤ϰΤߤǤ");
     }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.4.6
+//////////////////////////////////////////////////////////////////////////
+
+    /*! This is used to introduce a caller (or called-by) graph */
+    virtual QCString trCallerGraph()
+    {
+      // return "Here is the caller graph for this function:";
+      return decode("ƽФ:");
+    }
+
+    /*! This is used in the documentation of a file/namespace before the list
+     *  of documentation blocks for enumeration values
+     */
+    virtual QCString trEnumerationValueDocumentation()
+    { return decode(""); }
+
 };
 #endif
